package premium_store.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import premium_store.model.Client;
import premium_store.model.Tank;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.util.List;

@Service
public class ClientService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public Client addClient(String newNickName, String newEmail, Integer newBallance) {
        if (!StringUtils.hasText(newNickName) || !StringUtils.hasText(newEmail)) {
            throw new IllegalArgumentException("NickName or newEmail is zero or empty");
        }

        final Client client = new Client(newNickName, newEmail, newBallance);
        em.persist(client);

        return client;
    }

    @Transactional(readOnly = true)
    public Client findClient(Long id) {
        final Client Client = em.find(Client.class, id);

        if (Client == null) {
            throw new EntityNotFoundException(String.format("Client with id [%s] is not found", id));
        }

        return Client;
    }

    @Transactional(readOnly = true)
    public List<Client> findAllClients() {
        return em.createQuery("select c from Client c", Client.class)
                .getResultList();
    }

    @Transactional
    public Client updateClient(Long id, String newNickName, String newEmail, Integer newBalance, List<Tank> newTanks) {
        if (id <= 0) {
            throw new IllegalArgumentException("Invalid id");
        }

        final Client currentClient = findClient(id);

        if (StringUtils.hasText(newNickName)){
            currentClient.setNickName(newNickName);
        }

        if(StringUtils.hasText(newEmail)){
            currentClient.setEmail(newEmail);
        }

        if(newBalance != null){
            currentClient.setBalance(newBalance);
        }

        if(newTanks != null){
            currentClient.setTanks(newTanks);
        }

        return em.merge(currentClient);
    }

    @Transactional
    public Client deleteClient(Long id) {
        final Client currentClient = findClient(id);
        em.remove(currentClient);

        return currentClient;
    }

    //прямой sql-запрос на удаление всех записей в таблице
    @Transactional
    public void deleteAllClients() {
        em.createQuery("Delete from Client").executeUpdate();
    }
}
